/*  Gtk+ Widget Factory
 *  Copyright (C) 1999-2000  Jeroen Benckhuijsen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "gwf_lib.h"
#include "gwf_error.h"
#include "gwf_project.h"
#include "gwf_source.h"
#include "gwf_treedata.h"
#include "gwf_format.h"
#include "gwf_projlist.h"
#include "gwf_source_basic.h"
#include "gwf_widpropbasic.h"
#include "gwf_gpl.h"
#include "gwf_plugin.h"
#include "gwf_pixmaps.h"
#include "gwf_source_tools.h"
#include "gwf_xmlio.h"
#include "gwf_license.h"
#include "gwf_projtreebasic.h"
#include "gwf_xmlio_basic.h"
#include "gwf_projdata.h"
#include "gwf_settings.h"
#include "gwf_tools.h"
#include <gtk/gtk.h>
#include <sys/stat.h>

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifndef PACKAGE_LIB_DIR
#  define PACKAGE_LIB_DIR	"/usr/local/lib/gwf"
#  warning PACKAGE_LIB_DIR not defined using default.
#endif

#define USER_LIB_DIR		"/lib/gwf"

#ifdef HAVE_DIRENT_H
#  include <dirent.h>
#endif

#ifdef G_OS_WIN32
#  include <direct.h>
#  include <io.h>
#  define mkdir(p,m) _mkdir(p)
#endif

#define	N_ERR			34

static GwfErrData		std_err_msg[N_ERR] =	{
							  {"General : No project",GWF_ERROR,"There is no current project"},
							  {"General : Save : No filename",GWF_WARNING,"No filename selected"},
							  {"General : Load : no XML",GWF_ERROR,"The file could not be read"},
							  {"General : Load : Invalid XML",GWF_ERROR,"The specified file is not a valid Gwf file"},
							  {"General : Widget exists",GWF_ERROR,"A widget with that name already exists"},
							  {"General : New project",GWF_HINT,"New project created"},
							  {"General : Project loaded",GWF_HINT,"The project was loaded"},
							  {"General : Project saved",GWF_HINT,"The project was saved"},
							  {"General : Project deleted",GWF_HINT,"The project was deleted"},
							  {"General : Source written",GWF_HINT,"Source code written for the project"},
							  {"General : Source write : No C-file",GWF_ERROR,"The output c-file wasn't specified"},
							  {"General : Source write : No H-file",GWF_ERROR,"The output h-file wasn't specified"},
							  {"General : Source write : Check error",GWF_ERROR,"Some properties are invalid or not complete"},
							  {"General : Source write : Backup error",GWF_WARNING,"Couldn't create the backup source files"},
							  {"General : Source write : Can't open H-file",GWF_ERROR,"The h-file could not be opened for writing"},
							  {"General : Source write : Can't open C-file",GWF_ERROR,"The c-file could not be opened for writing"},
							  {"General : Source write : Can't write H-file",GWF_ERROR,"Couldn't write to the h-file (Disk full?"},
							  {"General : Source write : Can't write C-file",GWF_ERROR,"Couldn't write to the c-file (Disk full?"},
						  	  {"Format : Save : Can't write",GWF_ERROR,"Can't save the project to disk (Disk full?)"},
						  	  {"Format : Save : Can't open",GWF_ERROR,"Can't open the project file for saving"},
						  	  {"Format : Load : Can't open",GWF_ERROR,"Can't open the project file for loading"},
							  {"Plugins : Duplicate plugin",GWF_ERROR,"Can't load plugin: A plugin with this name already exists"},
							  {"Plugins : Invalid plugin",GWF_ERROR,"Can't load plugin: The file was not valid"},
							  {"Plugins : No plugin dir",GWF_ERROR,"The plugin directory does not exist"},
							  {"Source : Basic : No widname",GWF_ERROR,"The widget name was not set"},
							  {"Source : Basic : No parent",GWF_ERROR,"The parent widget was not set"},
							  {"XmlIO : Invalid Href",GWF_ERROR,"The url of the plugin doesn't match : %s"},
							  {"XmlIO : Save : Conversion error",GWF_ERROR,"An error occured while converting the project to XML"},
							  {"XmlIO : Unknown loader",GWF_WARNING,"Can't find plugin, download it at %s"},
							  {"XmlIO : Wrong version",GWF_WARNING,"The loaded plugin version is to low, download the new version at %s"},
							  {"XmlIO : Invalid file",GWF_ERROR,"The file is not a valid Gwf Project"},
							  {"XmlIO : Program version",GWF_ERROR,"The program version is to low, download a new version at http://gwf.sourceforge.net"},
							  {"XmlIO : Unknown tag",GWF_ERROR,"Unknown tag encountered in the file: %s"},
							  {"XmlIO : No name",GWF_ERROR,"The project file didn't contain a name for the widget"},
							};


static void		gwf_create_data 		(GwfData **data);
static gint		gwf_register_errors		(GwfData *data);
static gint		gwf_register_projectio		(GwfData *data);
static gint		gwf_register_gui		(GwfData *data);
static gint		gwf_create_user_dir		(GwfData *data);
static gint		gwf_load_plugins		(GwfData *data);

gint
gwf_data_init (GwfData **data,GwfErrorFunc error,gpointer errdata)
{
  g_return_val_if_fail(data != NULL,0);
  g_return_val_if_fail(error != NULL,0);

  /****************************/
  /* Check for module support */
  /****************************/
  if (!g_module_supported()) return 0;

  /***************************/
  /* Creation data stuctures */
  /***************************/
  gwf_create_data(data);

  /*************************************/
  /*  Register standard error messages */
  /*************************************/
  if (!gwf_register_errors(*data)) return 0;

  gwf_error_set_func((*data)->error,error,errdata);

  /*******************************/
  /* Register standard functions */
  /*******************************/
  if (!gwf_register_projectio(*data)) return 0;
  if (!gwf_register_gui(*data)) return 0;

  /****************/
  /* Load plugins */
  /****************/
  if (!gwf_create_user_dir(*data)) return 0;
  if (!gwf_load_plugins(*data)) return 0;

  return 1;
}

void
gwf_data_deinit	(GwfData *data)
{
  g_return_if_fail(data != NULL);

  /***************************/
  /* Free all allocated data */
  /***************************/
  gtk_object_destroy(GTK_OBJECT(data->projlist));
  gwf_settingslist_free(data->settings);
  gwf_xmlio_free(data->xmlio);
  gwf_treedata_list_free(data->treedatas);
  gwf_source_write_free(data->source);
  gwf_liclist_free(data->liclist);
  gwf_pluginlist_free(data->plugins);
  gwf_error_free(data->error);
}

gint
gwf_data_add_plugin (GwfData *data,GwfPlugin *plugin)
{
  g_return_val_if_fail(data != NULL,0);
  g_return_val_if_fail(plugin != NULL,0);

  if (!gwf_pluginlist_add_plugin(data->plugins,plugin))
    return 0;

  gwf_plugin_reg_errors(plugin,data->error);
  gwf_plugin_reg_settings(plugin,data->settings);
  gwf_plugin_reg_xmlio(plugin,data->xmlio);
  gwf_plugin_reg_treedata(plugin,data->treedatas);
  gwf_plugin_reg_source(plugin,data->source);

  return 1;
}

static void
gwf_create_data (GwfData **data)
{
  GwfData	*tmp;

  *data = g_malloc(sizeof(GwfData));
  tmp = *data;

  tmp->projlist = gwf_projlist_new();
  tmp->settings = gwf_settingslist_new();
  tmp->xmlio = gwf_xmlio_new();
  tmp->plugins = gwf_pluginlist_new();
  tmp->treedatas = gwf_treedata_list_new();
  tmp->source = gwf_source_write_new();
  tmp->liclist = gwf_liclist_new();
  tmp->error = gwf_error_new();
}

static gint
gwf_register_errors (GwfData *data)
{
  gint		i;

  for (i = 0 ; i < N_ERR; i++)
  {
    if (!gwf_error_add_error(data->error,std_err_msg[i].name,std_err_msg[i].type,std_err_msg[i].message))
      return 0;
  }

  return 1;
}

static gint
gwf_register_projectio(GwfData *data)
{
  GwfLicense	*license;

  /* Standard xml load & save functions */
  if (!gwf_xmlio_add_loader(data->xmlio,"Basic",XML_FILE_VERSION,XML_HREF,gwf_xmlio_load_basic))
    return 0;
  if (!gwf_xmlio_add_saver(data->xmlio,"Basic",XML_FILE_VERSION,XML_HREF,gwf_xmlio_save_basic))
    return 0;

  /* Standard source writing functions */
  gwf_source_add_check(data->source,"Basic",gwf_source_basic_check);
  gwf_source_schedule_func(data->source,"Basic",INCLUDE,gwf_source_basic_write_includes_h,HFILE);
  gwf_source_schedule_func(data->source,"Basic",DEFINE,gwf_source_basic_write_cast_defs,HFILE);
  gwf_source_schedule_func(data->source,"Basic",TYPEDEF,gwf_source_basic_write_widget_tdef,HFILE);
  gwf_source_schedule_func(data->source,"Basic",TYPE,gwf_source_basic_write_widget,HFILE);
  gwf_source_schedule_func(data->source,"Basic",TYPE,gwf_source_basic_write_widget_class,HFILE);
  gwf_source_schedule_func(data->source,"Basic",FUNC_DECL,gwf_source_basic_write_func_decl_h,HFILE);
  gwf_source_schedule_func(data->source,"Basic",INCLUDE,gwf_source_basic_write_includes_c,CFILE);
  gwf_source_schedule_func(data->source,"Basic",FUNC_DECL,gwf_source_basic_write_func_decl_c,CFILE);
  gwf_source_schedule_func(data->source,"Basic",VARIABEL,gwf_source_basic_write_vars,CFILE);
  gwf_source_schedule_func(data->source,"Basic",FUNCTION,gwf_source_basic_write_funcs,CFILE);

  /* Standard License (GPL) */
  license = gwf_license_new(gpl_name);
  if (!license)
    return 0;
  gwf_license_set_text(license,gpl_text);
  gwf_liclist_insert(data->liclist,license);
  if (!gwf_liclist_set_std(data->liclist,gpl_name))
    return 0;

  return 1;
}

static gint
gwf_register_gui(GwfData *data)
{
  GwfSettings	*setting;
  GwfTreeData	*treedata;

  /* Standard widget properties settings windows */
  setting = gwf_settings_new("Basic");
  gwf_settings_set_create(setting,(GwfSettingsCreate)gwf_widpropbasic_new);
  if (!gwf_settingslist_add(data->settings,setting))
  {
    gwf_settings_free(setting);
    return 0;
  }

  /* Standard widget properties tree views */
  treedata = gwf_treedata_new("Basic");
  gwf_treedata_set_attach(treedata,gwf_projtree_basic_attach);
  if (!gwf_treedata_list_add(data->treedatas,treedata))
  {
    gwf_treedata_free(treedata);
    return 0;
  }

  return 1;
}

static gint
gwf_load_plugins(GwfData *data)
{
  #ifdef LOAD_SYS_PLUGINS
    gchar		*tmp,*dir;
  #endif
  #ifdef LOAD_USER_PLUGINS
    #ifndef LOAD_SYS_PLUGINS
      gchar		*tmp,*dir;
    #endif
  #endif

  #ifdef LOAD_SYS_PLUGINS
    tmp = g_get_home_dir();
    dir = g_strdup_printf("%s%s/plugins",tmp,USER_LIB_DIR);
    g_free(tmp);
    gwf_pluginlist_load_dir(data->plugins,dir,data->error);
    g_free(dir);
  #endif

  #ifdef LOAD_USER_PLUGINS
    dir = g_strdup_printf("%s/plugins",PACKAGE_LIB_DIR);
    gwf_pluginlist_load_dir(data->plugins,dir,data->error);
    g_free(dir);
  #endif

  gwf_pluginlist_reg_errors(data->plugins,data->error);
  gwf_pluginlist_reg_settings(data->plugins,data->settings);
  gwf_pluginlist_reg_xmlio(data->plugins,data->xmlio);
  gwf_pluginlist_reg_treedatas(data->plugins,data->treedatas);
  gwf_pluginlist_reg_source(data->plugins,data->source);
  gwf_pluginlist_connect_projlist(data->plugins,data->projlist);

  return 1;
}

static gint
gwf_create_user_dir (GwfData *data)
{
  #ifdef LOAD_USER_PLUGINS
  gchar		*tmp,*dir;
  DIR		*direc;

  tmp = g_get_home_dir();
  dir = g_strdup_printf("%s%s",tmp,"/lib");

  direc = opendir(dir);
  if (!direc)
  {
    g_message("Creating library directory in %s",dir);
    if (mkdir(dir,0755) < 0)
    {
      g_warning("Can't create library directory in %s, aborting...",dir);
      return 0;
    }
  }
  else
    closedir(direc);

  tmp = dir;
  dir = g_strdup_printf("%s%s",tmp,"/gwf");
  g_free(tmp);

  direc = opendir(dir);
  if (!direc)
  {
    g_message("Creating Gwf directory in %s",dir);
    if (mkdir(dir,0755) < 0)
    {
      g_warning("Can't create Gwf directory in %s, aborting...",dir);
      return 0;
    }
  }
  else
    closedir(direc);

  tmp = dir;
  dir = g_strdup_printf("%s%s",tmp,"/plugins");
  g_free(tmp);

  direc = opendir(dir);
  if (!direc)
  {
    g_message("Creating Gwf plugins directory in %s",dir);
    if (mkdir(dir,0755) < 0)
    {
      g_warning("Can't create Gwf plugins directory in %s, aborting...",dir);
      return 0;
    }
  }
  else
    closedir(direc);

  #endif
  return 1;
}
